// program img2tif
// usage -- %img2tif input output from to bits");

#include <stdio.h>
#include <stdlib.h>
#include <string.h>
#include <math.h>
#include "cell.h"
#include "sif.h"

#define INTEL

#define HiPic_Header_Size 64

struct HiPic_Header
{
	char					head[2];
	unsigned short			comment_length;
	unsigned short			width;
	unsigned short			height;
	unsigned short			x_offset;
	unsigned short			y_offset;
	unsigned short			type;
	char					reserved[63 - 13];
	char					*comment;
};
typedef struct HiPic_Header Header;

unsigned short	*data;

static void	Error(msg)
char		*msg;
{
	fputs(msg,stderr); fputc('\n',stderr); exit(1);
}


/*----------------------------------------------------------------------*/
int read_hipic(fname, h)
char	*fname;
Header	*h;
{
		short		x_size, y_size;
		int		j;
		FILE	*fi;

//open input files
		if((fi = fopen(fname,"rb")) == NULL)
		{
			printf("can not open %s for input\n", fname);
			free(h->comment); 
			return(-10);
		}
// read comment and image data from file1
		if (fread(h, sizeof(char), HiPic_Header_Size, fi) != HiPic_Header_Size)
		{
			fprintf(stderr, "EOF in %s (header)\n", fname);
			free(h->comment); 
			return(-1);
		}
		if (strncmp(h->head, "IM", 2))
		{
			fclose(fi); 
			free(h->comment); 
			fprintf(stderr, "This is not HiPIc image file.\n"); 
			fprintf(stderr, "File name = %s\n", fname); 
			return(-11);
		}

		h->comment = (char *) malloc(h->comment_length + 1);
		if (h->comment == NULL)
		{
			fclose(fi); 
			free(h->comment); 
			fprintf(stderr, "Connot allocate memory (HiPic_Header.comment)\n"); 
			fprintf(stderr, "File name = %s\n", fname); 
			return(-12);
		}
		if (fread(h->comment, sizeof(char), h->comment_length, fi) != h->comment_length)
		{ 
			fclose(fi); 
			free(h->comment); 
			fprintf(stderr, "EOF found (comment)\n"); 
			fprintf(stderr, "File name = %s\n", fname); 
			return(-13);
		}

		h->comment[h->comment_length] = '\0';

		x_size = h->width;
		y_size = h->height;

		data = (unsigned short *) malloc(x_size * y_size * sizeof(unsigned short));
		if ((j = fread(data, sizeof(unsigned short), x_size*y_size, fi)) != x_size*y_size)
		{
			fclose(fi);
			free(h->comment); 
			fprintf(stderr, " Error reading %s at %d\n", fname, j);
			fprintf(stderr, "File name = %s\n", fname); 
			return(1);
		}

// check for correct reading of HiPic file
//printf("-------------------------------\n");
//printf("%s\n", fname);
//printf("%s\n", h->head);
//printf("%d\n", h->comment_length);
//printf("%d\n", h->width);
//printf("%d\n", h->height);
//printf("%d\n", h->x_offset);
//printf("%d\n", h->y_offset);
//printf("%d\n", h->type);
//printf("%s\n", h->reserved);
//printf("%s\n", h->comment);
//printf("-------------------------------\n");

		fclose(fi);
		return(0);
	}

/*----------------------------------------------------------------------*/

int	main(argc,argv)
int	argc;
char	**argv;
{
	char	*comm = NULL;

	int		i, j;

	short	Nx, Ny;
	unsigned short	data_max, data_min;
	Cell	**cell;
	int		y,x;
	long	val;

	short	frm, to;
	short	nbit;

	Header	h;

	if (argc!=6) Error("usage : img2tif input output from to bits");

	if ((i = read_hipic(argv[1], &h)) != 0){
		printf("something wrong -- return value is %d", i);
		exit(-1);
	}
	free(h.comment);

	frm = atoi(argv[3]);
	to  = atoi(argv[4]);
	nbit= atoi(argv[5]);
	Nx = h.width;
	Ny = h.height;

	data_max = 0;
	data_min = 32000;
	for (i=0; i<Nx*Ny;i++){
		if(data_max<*(data+i)) data_max=*(data+i);
		if(data_min>*(data+i)) data_min=*(data+i);
//if((j=(int)fmod(i,10)) == 0) printf("%d\t%d\n",i,*(data+i));
	}

// wrong input parameter
	if(frm<0) frm = data_min;
	if(frm>data_max) Error("too big number.(from)");
	if(to<1 || to>65535) to=data_max;
	if(nbit!=8 && nbit!=16) Error("wrong bit number.select 8 or 16");


	printf("Image size:\n");
	printf("\t%d x %d\n", Nx, Ny);
	printf("MAX and min:\n");
	printf("\t%d to %d\n\n", data_max, data_min);
	if (nbit == 16) printf("img file(14 bits) is converted into 16bits TIFF.\n");
	if (nbit == 8){
		printf("img file(14 bits) is converted into 8bits TIFF.\n");
		printf("from %d to %d\n", frm, to);
	}

// initialize cell
	if ((cell=(Cell **)malloc(Ny*sizeof(*cell)))==NULL)
	    Error("cell memory allocation error (1).");

	for (y=0; y<Ny; y++) {
	    if ((cell[y]=(Cell *)malloc(Nx*sizeof(**cell)))==NULL)
		Error("cell memory allocation error (2).");
	}

	if (nbit==16){
		j = 0;
		for (y=0; y<Ny; y++){
			for (x=0; x<Nx; x++){
				cell[y][x]=(unsigned short)(*(data+j)*1.0/1.0);
				j=j+1;
			}
		}
	    if ((comm=(char *)malloc(35))==NULL)
			Error("comment memory allocation error.");
		comm="14bit ITEX file to 16bit TIFF file";
	}
	else if (nbit==8){
		j = 0;
		for (y=0; y<Ny; y++){
			for (x=0; x<Nx; x++){
				val = (long)((double)255*((double)*(data+j)-(double)frm)/((double)to-(double)frm));
				if(val < 1) val = 0;
				if(val > 254) val = 255;
				cell[y][x]=(unsigned short)val;
				j=j+1;
			}
		}
	    if ((comm=(char *)malloc(55))==NULL)
			Error("comment memory allocation error.");
		sprintf(comm,"14bit ITEX file to 8bit TIFF file, from %5d to %5d",frm, to);
	}

	StoreTiffFile(argv[2],Nx,Ny,nbit,cell,comm);
	printf("Store %s\n",argv[2]);

	return 0;
}
